/*
 * Sixth Data. Copyright ©2012-2016, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 */

package eu.svjatoslav.sixth.data.store.file;

import java.io.IOException;

class EntryRecord implements Comparable<EntryRecord> {
    public static final int ENTRY_RECORD_LENGTH = 12;
    final int id;
    long location = 0;
    int length = 0;

    public EntryRecord(final FileDataStore dataStore, final int id)
            throws IOException {

        this.id = id;

        final long entryRecordLocation = dataStore.entryAllocationTable
                .getEntryRecordLocation(id);

        location = dataStore.readLong(entryRecordLocation);

        if (isUsed())
            length = dataStore.readInt(entryRecordLocation + 8);
    }

    public EntryRecord(final int id, final long location, final int length) {
        this.id = id;
        this.location = location;
        this.length = length;
    }

    public void clear() {
        location = 0;
        length = 0;
    }

    @Override
    public boolean equals(final Object o) {
        if (o == null) return false;
        return o instanceof EntryRecord && compareTo((EntryRecord) o) == 0;
    }

    @Override
    public int compareTo(final EntryRecord o) {
        if (location < o.location)
            return -1;
        if (location > o.location)
            return 1;

        if (id < o.id)
            return -1;
        if (id > o.id)
            return 1;

        return 0;
    }

    @Override
    public int hashCode() {
        int result = id;
        result = 31 * result + (int) (location ^ (location >>> 32));
        return result;
    }

    public boolean isUsed() {
        return location != 0;

    }

    public void save(final FileDataStore dataStore) throws IOException {

        final long entryRecordLocation = dataStore.entryAllocationTable
                .getEntryRecordLocation(id);

        dataStore.writeLong(entryRecordLocation, location);
        dataStore.writeInt(entryRecordLocation + 8, length);
    }
}
