/*
 * Sixth - System for data storage, computation, exploration and interaction.
 * Copyright ©2012-2016, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 */

package eu.svjatoslav.sixth.datastore;

import eu.svjatoslav.sixth.data.store.DataStore;
import eu.svjatoslav.sixth.data.store.file.FileDataStore;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

public class DataStoreImplTest {

    public static final String UTF_8 = "UTF-8";
    File dataSourceFile;

    DataStore dataStore;

    @Before
    public void setUp() throws Exception {
        dataSourceFile = new File("test.byar");
        if (dataSourceFile.exists())
            if (!dataSourceFile.delete())
                throw new RuntimeException("Cannot delete file: " + dataSourceFile.getCanonicalPath());

        dataStore = new FileDataStore(dataSourceFile);
    }

    @Test
    public void testBasicStorageAndRetrieval() throws IOException {

        final int record1 = dataStore.createRecord("See on test".getBytes(UTF_8));
        final int record2 = dataStore.createRecord("another test".getBytes(UTF_8));

        Assert.assertEquals(0, record1);
        Assert.assertEquals(1, record2);

        Assert.assertEquals("another test", new String(dataStore.readRecord(record2),
                UTF_8));
        Assert.assertEquals("See on test", new String(dataStore.readRecord(record1),
                UTF_8));

        dataStore.updateRecord(record1, "!!!".getBytes(UTF_8));

        Assert.assertEquals("!!!", new String(dataStore.readRecord(record1), UTF_8));

        dataStore.updateRecord(record1, dataStore.readRecord(record2));

        Assert.assertEquals("another test", new String(dataStore.readRecord(record1),
                UTF_8));

        dataStore.close();
    }

    @Test
    public void testDefragmentation() throws IOException {
        final int record1 = dataStore.createRecord("1111".getBytes(UTF_8));
        final int record2 = dataStore.createRecord("22".getBytes(UTF_8));
        final int record3 = dataStore.createRecord("3333333".getBytes(UTF_8));
        final int record4 = dataStore.createRecord("4".getBytes(UTF_8));

        ((FileDataStore) dataStore).vacuum(0);

        final int record5 = dataStore.createRecord("555".getBytes());

        dataStore.deleteRecord(record1);

        ((FileDataStore) dataStore).vacuum(0);

        final int record6 = dataStore.createRecord("66".getBytes());

        dataStore.close();
    }

    @Test
    public void testEntriesTableIncreasing() throws IOException {
        // dataStore.createRecord(String.valueOf("lalalala").getBytes());

        for (int j = 0; j < 10; j++) {
            final ArrayList<Integer> ids = new ArrayList<>();

            for (int i = 0; i < 100; i++) {
                final int id = dataStore.createRecord(String.valueOf(
                        "____________" + i).getBytes());

                ids.add(id);
            }

            for (final int id : ids)
                dataStore.deleteRecord(id);

            ((FileDataStore) dataStore).vacuum(0);
        }
    }

}
